@@ -1,24 +1,20 @@
-import imp
+import sys
 from inspect import getmembers, getmro, isclass
 from pkgutil import iter_modules
-import sys
-
-from venusian.compat import (
-    is_nonstr_iter,
-    INT_TYPES,
-    )
 
 from venusian.advice import getFrameInfo
+from venusian.compat import compat_find_loader
+
+ATTACH_ATTR = "__venusian_callbacks__"
+LIFTONLY_ATTR = "__venusian_liftonly_callbacks__"
 
-ATTACH_ATTR = '__venusian_callbacks__'
-LIFTONLY_ATTR = '__venusian_liftonly_callbacks__'
 
 class Scanner(object):
     def __init__(self, **kw):
         self.__dict__.update(kw)
 
     def scan(self, package, categories=None, onerror=None, ignore=None):
-        """ Scan a Python package and any of its subpackages.  All
+        """Scan a Python package and any of its subpackages.  All
         top-level objects will be considered; those marked with
         venusian callback attributes related to ``category`` will be
         processed.
@@ -39,7 +35,7 @@ class Scanner(object):
         happen during its code importing process, including
         :exc:`ImportError`.  If you use a custom ``onerror`` callback, you
         can change this behavior.
-        
+
         Here's an example ``onerror`` callback that ignores
         :exc:`ImportError`::
 
@@ -101,14 +97,16 @@ class Scanner(object):
 
         A string or callable alone can also be passed as ``ignore`` without a
         surrounding list.
-        
+
         .. versionadded:: 1.0a3
            the ``ignore`` argument
         """
 
         pkg_name = package.__name__
 
-        if ignore is not None and not is_nonstr_iter(ignore):
+        if ignore is not None and (
+            isinstance(ignore, str) or not hasattr(ignore, "__iter__")
+        ):
             ignore = [ignore]
         elif ignore is None:
             ignore = []
@@ -116,12 +114,12 @@ class Scanner(object):
         # non-leading-dotted name absolute object name
         str_ignores = [ign for ign in ignore if isinstance(ign, str)]
         # leading dotted name relative to scanned package
-        rel_ignores = [ign for ign in str_ignores if ign.startswith('.')]
+        rel_ignores = [ign for ign in str_ignores if ign.startswith(".")]
         # non-leading dotted names
-        abs_ignores = [ign for ign in str_ignores if not ign.startswith('.')]
+        abs_ignores = [ign for ign in str_ignores if not ign.startswith(".")]
         # functions, e.g. re.compile('pattern').search
         callable_ignores = [ign for ign in ignore if callable(ign)]
-        
+
         def _ignore(fullname):
             for ign in rel_ignores:
                 if fullname.startswith(pkg_name + ign):
@@ -135,10 +133,8 @@ class Scanner(object):
                     return True
             return False
 
-
         def invoke(mod_name, name, ob):
-
-            fullname = mod_name + '.' + name
+            fullname = mod_name + "." + name
 
             if _ignore(fullname):
                 return
@@ -188,7 +184,7 @@ class Scanner(object):
                             # this module but were not actually defined there
                             continue
                         callback(self, name, ob)
-                except ValueError:
+                except ValueError:  # pragma: nocover
                     continue
 
         for name, ob in getmembers(package):
@@ -197,51 +193,47 @@ class Scanner(object):
             # subpackages
             invoke(pkg_name, name, ob)
 
-        if hasattr(package, '__path__'): # package, not module
-            results = walk_packages(package.__path__, package.__name__+'.',
-                                    onerror=onerror, ignore=_ignore)
+        if hasattr(package, "__path__"):  # package, not module
+            results = walk_packages(
+                package.__path__,
+                package.__name__ + ".",
+                onerror=onerror,
+                ignore=_ignore,
+            )
 
             for importer, modname, ispkg in results:
-                loader = importer.find_module(modname)
-                if loader is not None: # happens on pypy with orphaned pyc
+                loader = compat_find_loader(importer, modname)
+                if loader is not None:  # happens on pypy with orphaned pyc
                     try:
-                        if hasattr(loader, 'etc'):
-                            # python < py3.3
-                            module_type = loader.etc[2]
-                        else: # pragma: no cover
-                            # py3.3b2+ (importlib-using)
-                            module_type = imp.PY_SOURCE
-                            get_filename = getattr(loader, 'get_filename', None)
-                            if get_filename is None:
-                                get_filename = loader._get_filename
-                            try:
-                                fn = get_filename(modname)
-                            except TypeError:
-                                fn = get_filename()
-                            if fn.endswith(('.pyc', '.pyo', '$py.class')):
-                                module_type = imp.PY_COMPILED
-                        # only scrape members from non-orphaned source files
-                        # and package directories
-                        if module_type in (imp.PY_SOURCE, imp.PKG_DIRECTORY):
-                            # NB: use __import__(modname) rather than
-                            # loader.load_module(modname) to prevent
-                            # inappropriate double-execution of module code
-                            try:
-                                __import__(modname)
-                            except Exception:
-                                if onerror is not None:
-                                    onerror(modname)
-                                else:
-                                    raise
-                            module = sys.modules.get(modname)
-                            if module is not None:
-                                for name, ob in getmembers(module, None):
-                                    invoke(modname, name, ob)
+                        get_filename = getattr(loader, "get_filename", None)
+                        if get_filename is None:  # pragma: nocover
+                            get_filename = loader._get_filename
+                        try:
+                            fn = get_filename(modname)
+                        except TypeError:  # pragma: nocover
+                            fn = get_filename()
+
+                        # NB: use __import__(modname) rather than
+                        # loader.load_module(modname) to prevent
+                        # inappropriate double-execution of module code
+                        try:
+                            __import__(modname)
+                        except Exception:
+                            if onerror is not None:
+                                onerror(modname)
+                            else:
+                                raise
+                        module = sys.modules.get(modname)
+                        if module is not None:
+                            for name, ob in getmembers(module, None):
+                                invoke(modname, name, ob)
                     finally:
-                        if  ( hasattr(loader, 'file') and
-                              hasattr(loader.file,'close') ):
+                        if hasattr(loader, "file") and hasattr(
+                            loader.file, "close"
+                        ):  # pragma: nocover
                             loader.file.close()
 
+
 class AttachInfo(object):
     """
     An instance of this class is returned by the
@@ -277,11 +269,13 @@ class AttachInfo(object):
       representing the context of the venusian decorator used.  Eg.
       ``('/home/chrism/projects/venusian/tests/test_advice.py', 81,
       'testCallInfo', 'add_handler(foo, bar)')``
-      
+
     """
+
     def __init__(self, **kw):
         self.__dict__.update(kw)
 
+
 class Categories(dict):
     def __init__(self, attached_to):
         super(dict, self).__init__()
@@ -292,12 +286,13 @@ class Categories(dict):
         self.lifted = False
 
     def attached_to(self, mod_name, name, obj):
-        if isinstance(self.attached_id, INT_TYPES):
+        if isinstance(self.attached_id, int):
             return self.attached_id == id(obj)
         return self.attached_id == (mod_name, name)
 
+
 def attach(wrapped, callback, category=None, depth=1, name=None):
-    """ Attach a callback to the wrapped object.  It will be found
+    """Attach a callback to the wrapped object.  It will be found
     later during a scan.  This function returns an instance of the
     :class:`venusian.AttachInfo` class.
 
@@ -309,20 +304,20 @@ def attach(wrapped, callback, category=None, depth=1, name=None):
     determine if decorations of a method should be inherited or overridden.
     """
 
-    frame = sys._getframe(depth+1)
+    frame = sys._getframe(depth + 1)
     scope, module, f_locals, f_globals, codeinfo = getFrameInfo(frame)
-    module_name = getattr(module, '__name__', None)
-    wrapped_name = getattr(wrapped, '__name__', None)
+    module_name = getattr(module, "__name__", None)
+    wrapped_name = getattr(wrapped, "__name__", None)
     class_name = codeinfo[2]
 
-    liftid = '%s %s' % (wrapped_name, name)
-    
-    if scope == 'class':
+    liftid = "%s %s" % (wrapped_name, name)
+
+    if scope == "class":
         # we're in the midst of a class statement
         categories = f_locals.get(ATTACH_ATTR, None)
         if categories is None or not categories.attached_to(
             module_name, class_name, None
-            ):
+        ):
             categories = Categories((module_name, class_name))
             f_locals[ATTACH_ATTR] = categories
         callbacks = categories.setdefault(category, [])
@@ -330,7 +325,7 @@ def attach(wrapped, callback, category=None, depth=1, name=None):
         categories = getattr(wrapped, ATTACH_ATTR, None)
         if categories is None or not categories.attached_to(
             module_name, wrapped_name, wrapped
-            ):
+        ):
             # if there aren't any attached categories, or we've retrieved
             # some by inheritance, we need to create new ones
             categories = Categories(wrapped)
@@ -346,9 +341,10 @@ def attach(wrapped, callback, category=None, depth=1, name=None):
         globals=f_globals,
         category=category,
         codeinfo=codeinfo,
-        )
+    )
 
-def walk_packages(path=None, prefix='', onerror=None, ignore=None):
+
+def walk_packages(path=None, prefix="", onerror=None, ignore=None):
     """Yields (module_loader, name, ispkg) for all modules recursively
     on path, or, if path is None, all accessible modules.
 
@@ -384,13 +380,12 @@ def walk_packages(path=None, prefix='', onerror=None, ignore=None):
     """
 
     def seen(p, m={}):
-        if p in m: # pragma: no cover
+        if p in m:  # pragma: no cover
             return True
         m[p] = True
 
     # iter_modules is nonrecursive
     for importer, name, ispkg in iter_modules(path, prefix):
-
         if ignore is not None and ignore(name):
             # if name is a package, ignoring here will cause
             # all subpackages and submodules to be ignored too
@@ -408,12 +403,12 @@ def walk_packages(path=None, prefix='', onerror=None, ignore=None):
                     raise
             else:
                 yield importer, name, ispkg
-                path = getattr(sys.modules[name], '__path__', None) or []
+                path = getattr(sys.modules[name], "__path__", None) or []
 
                 # don't traverse path items we've seen before
                 path = [p for p in path if not seen(p)]
 
-                for item in walk_packages(path, name+'.', onerror, ignore):
+                for item in walk_packages(path, name + ".", onerror, ignore):
                     yield item
         else:
             yield importer, name, ispkg
@@ -436,13 +431,13 @@ class lift(object):
 
           @venusian_decorator()
           def jump(self): pass
-              
+
       @lift()
       class Sub(Super):
           def boo(self): pass
 
           def hiss(self): pass
-          
+
           @venusian_decorator()
           def smack(self): pass
 
@@ -467,7 +462,7 @@ class lift(object):
       class Sub(Super):
 
           def boo(self): pass
-          
+
           @venusian_decorator()
           def hiss(self): pass
 
@@ -479,8 +474,9 @@ class lift(object):
     The ``lift`` decorator takes a single argument named 'categories'.  If
     supplied, it should be a tuple of category names.  Only decorators
     in this category will be lifted if it is suppled.
-          
+
     """
+
     def __init__(self, categories=None):
         self.categories = categories
 
@@ -488,11 +484,11 @@ class lift(object):
         if not isclass(wrapped):
             raise RuntimeError(
                 '"lift" only works as a class decorator; you tried to use '
-                'it against %r' % wrapped
-                )
+                "it against %r" % wrapped
+            )
         frame = sys._getframe(1)
         scope, module, f_locals, f_globals, codeinfo = getFrameInfo(frame)
-        module_name = getattr(module, '__name__', None)
+        module_name = getattr(module, "__name__", None)
         newcategories = Categories(wrapped)
         newcategories.lifted = True
         for cls in getmro(wrapped):
@@ -509,9 +505,9 @@ class lift(object):
                     for cb, _, liftid, cscope in category:
                         append = True
                         toappend = (cb, module_name, liftid, cscope)
-                        if cscope == 'class':
+                        if cscope == "class":
                             for ncb, _, nliftid, nscope in callbacks:
-                                if (nscope == 'class' and liftid == nliftid):
+                                if nscope == "class" and liftid == nliftid:
                                     append = False
                         if append:
                             newcallbacks.append(toappend)
@@ -519,10 +515,11 @@ class lift(object):
                     newcategories[cname] = newcategory
                 if attached_categories.lifted:
                     break
-        if newcategories: # if it has any keys
+        if newcategories:  # if it has any keys
             setattr(wrapped, ATTACH_ATTR, newcategories)
         return wrapped
-        
+
+
 class onlyliftedfrom(object):
     """
     A class decorator which marks a class as 'only lifted from'.  Decorations
@@ -531,7 +528,7 @@ class onlyliftedfrom(object):
     ``lift`` class decorator will use the superclass decoration callbacks.
 
     For example::
-    
+
       from venusian import lift, onlyliftedfrom
       from somepackage import venusian_decorator
 
@@ -547,19 +544,20 @@ class onlyliftedfrom(object):
       class Sub(Super):
 
           def boo(self): pass
-          
+
           def hiss(self): pass
 
     Only two decorator callbacks will be run: the ones attached to Sub.boo and
     Sub.hiss.  The inherited decorators on Super.boo and Super.hiss will be
     not be registered.
     """
+
     def __call__(self, wrapped):
         if not isclass(wrapped):
             raise RuntimeError(
                 '"onlyliftedfrom" only works as a class decorator; you tried '
-                'to use it against %r' % wrapped
-                )
+                "to use it against %r" % wrapped
+            )
         cats = getattr(wrapped, ATTACH_ATTR, None)
         class_name = wrapped.__name__
         module_name = wrapped.__module__
@@ -571,4 +569,3 @@ class onlyliftedfrom(object):
         delattr(wrapped, ATTACH_ATTR)
         setattr(wrapped, LIFTONLY_ATTR, cats)
         return wrapped
-        
